__author__ = "o-murphy"
__copyright__ = ("",)

__credits__ = ["O-Murphy"]
__version__ = "1.0.11"

__all__ = ['profile', 'atmosphere', 'bmath', 'drag', 'drag_tables', 'projectile', 'shot_parameters',
           'trajectory_calculator', 'trajectory_data', 'weapon', 'wind', 'multiple_bc']
